﻿#		File:			XenAppAudit.XA6.ps1
#		Description:	Audit XenApp Farm and write Access Database for Analysis
#		Requirements:	XenApp 6.0, XenApp PS SDK
#		Created by:		Andy Paul
#						www.paultechnologies.com
#
#		A comprehensive script recording farm, application, server, etc. properties.
#		Assumes C:\Program Files\XenAppTemp location but can be modified in code as needed.
#		For x64 systems, run PowerShell x86
#
$curlocation = Get-location
$curdir = $curlocation.path
$progpath = $curdir -replace("Scripts")
if($progpath.substring($progpath.length - 1, 1) -ne "\" ){$progpath += "\"}
Write-Host "Program Path :==  "  $progpath -ForegroundColor Green
$resultsPath = $progPath + "Results"
$scriptsPath = $progPath + "Scripts"
$transcriptFile = $resultsPath + "\Transcript_PS.txt"
Write-Host "Transcript File :==  " $transcriptfile -ForegroundColor Green
start-transcript -path $transcriptFile
#Add Citrix SDK Snapins, from Citrix.XenApp.Sdk.ps1
if (Get-PSSnapin Citrix.Common.Commands -ea 0)
{
	Write-Host "Citrix.Common.Commands snapin already loaded" -ForegroundColor Yellow
}
else
{
	Write-Host "Loading Citrix.Common.Commands snapin..." -ForegroundColor Yellow
	Add-PSSnapIn Citrix.Common.Commands
}

# If the snap-in is registered (i.e. this is a XenApp server), then load it
if (Get-PSSnapin Citrix.XenApp.Commands -ea 0)
{
	Write-Host "Citrix.XenApp.Commands snapin already loaded" -ForegroundColor Yellow
}
else
{
	if (Get-PSSnapIn "Citrix.XenApp.Commands" -registered -ea 0)
	{
		Write-Host "Loading Citrix.XenApp.Commands snapin..." -ForegroundColor Yellow
		Add-PSSnapin "Citrix.XenApp.Commands"
	}
}

if (Get-PSSnapin Citrix.Common.GroupPolicy -ea 0)
{
	Write-Host "Citrix.Common.GroupPolicy snapin already loaded" -ForegroundColor Yellow
}
else
{
	Write-Host "Loading Citrix.Common.GroupPolicy snapin..." -ForegroundColor Yellow
	Add-PSSnapIn Citrix.Common.GroupPolicy
}

#Import the remoting commands scripts module
if (Get-Module Citrix.XenApp.Commands.Remoting)
{
	Write-Host "Citrix.XenApp.Commands.Remoting module already loaded" -ForegroundColor Yellow
}
else
{
	Write-Host "Loading Citrix.XenApp.Commands.Remoting module..." -ForegroundColor Yellow
	Import-Module -name Citrix.XenApp.Commands.Remoting
}


Write-Host
Write-Host
$AuditDate = Get-Date -Format G
Write-Host "-----------------------------------------------------------" -ForegroundColor Green
Write-Host "Starting Collection at"$AuditDate -ForegroundColor Green
Write-Host "-----------------------------------------------------------" -ForegroundColor Green
$AuditID = 0
$dbName = $resultsPath + "\XenAppAudit_be.mdb"

$localhost=[environment]::MachineName
$localuser=[environment]::UserName

#Write-Host "Press any key to continue ..." -ForegroundColor Blue
#$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")

$adOpenStatic = 3
$adLockOptimistic = 3
$connection = New-Object -com ADODB.Connection

$connection.Open("Provider=Microsoft.Jet.OLEDB.4.0;Data Source=$dbName")
$objRSAudit = New-Object -com ADODB.Recordset
$objRSAudit.Open("Select top 1 * from tblAudit", $connection, $adOpenStatic, $adLockOptimistic)
	$objRSAudit.AddNew()
	$objRSAudit.Fields.item('AuditServer').value = $localhost
	$objRSAudit.Fields.item('AuditDate').value = $AuditDate
	$objRSAudit.Fields.item('AuditUser').value = $localuser
	$objRSAudit.Update()
$objRSAudit.Close()

Write-Host
Write-Host "Collecting Farm Data..." 

$farm = Get-XAFarm
$farmname = $farm.farmname
Write-Host "     " $farmname  -ForegroundColor Gray

$qryString = "Select top 1 * from tblAudit where AuditDate = #" 
$qryString += $AuditDate
$qryString += "#" 

$objRSAudit.Open($qryString, $connection, $adOpenStatic, $adLockOptimistic)
	$objRSAudit.MoveFirst()
	$AuditID = $objRSAudit.Fields.item('AuditID').value
	$objRSAudit.Fields.item('AuditFarmName').value = $farmname
	$objRSAudit.Update()
$objRSAudit.Close()

$objRSFarm = New-Object -com ADODB.Recordset
$objRSFarm.Open("Select top 1 * from tblFarms", $connection, $adOpenStatic, $adLockOptimistic)
	$objRSFarm.AddNew()
	$objRSFarm.Fields.item('AuditID').value = $AuditID
	$objRSFarm.Fields.item('FarmName').value = $farmname
	$objRSFarm.Fields.item('FarmVersion').value = $farm.serverversion
	$objRSFarm.Update()
$objRSFarm.Close()

[string]$XenAppVersion = $farm.serverversion
Write-Host "     Farm Version:" $XenAppVersion
Write-Host "     Short Version:" $XenAppVersion.substring(0,3)

$qryString = "Select top 1 * from tblFarms where AuditID="
$qryString += $AuditID 
$qryString += " ORDER BY FarmID DESC" 

#Write-Host $qryString -ForegroundColor Yellow

$objRSFarm.Open($qryString , $connection, $adOpenStatic, $adLockOptimistic)
	$objRSFarm.MoveFirst()
	$FarmID = $objRSFarm.Fields.item('FarmID').value
$objRSFarm.Close()

#Write-Host "Press any key to continue ..." -ForegroundColor Blue
#$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")

Write-Host
Write-Host "Collecting Server Data..." 

$objRSServer = New-Object -com ADODB.Recordset
$objRSServer2 = New-Object -com ADODB.Recordset
$objRSPrinters = New-Object -com ADODB.Recordset
$objRSHF = New-Object -com ADODB.Recordset
$objRStblProcesses = New-Object -com ADODB.Recordset

$objRSServer.Open("Select top 1 * from tblServers", $connection, $adOpenStatic, $adLockOptimistic)
$Servers = Get-XAServer 
foreach($server in $servers){
		#This section shows information about the server name and the edition and version of XenApp installed
	
	$strComputer = $Server.ServerName
	Write-Host "     " $strComputer  -ForegroundColor Gray
	Write-Host "         - Real-time WMI Data..." -ForegroundColor DarkGray

	#check PING Status
	$ping = new-object system.net.networkinformation.ping
	$pingreturns = $ping.send($strComputer)
	$pingstatus = $pingreturns.status 
	Write-Host "                 Ping Status: " $pingstatus -ForegroundColor DarkGray
	if ($pingstatus -eq "Success")
	{	
	$colSettings = get-wmiobject -class "Win32_ComputerSystem" -namespace "root\CIMV2" -computername $strComputer
	Write-Host "                 Getting Win32_ComputerSystem WMI data"-ForegroundColor DarkGray
	foreach($objComputer in $colSettings){
		$strMFG = $objComputer.Manufacturer
		$strModel = $objComputer.Model
		$strMemory = $objComputer.TotalPhysicalMemory/1000000/1024
		$strMemory  = [int] $strMemory 
		$strProcs = $objComputer.NumberofLogicalProcessors
	}

	$colItems = get-wmiobject -class "Win32_Processor" -namespace "root\CIMV2" -computername $strComputer
	Write-Host "                 Getting Win32_Processor WMI data"-ForegroundColor DarkGray
	foreach($objItem in $colItems){$strClockSpeed = $objItem.MaxClockSpeed}


	$colItems = get-wmiobject -class "Win32_PerfFormattedData_PerfOS_Processor WHERE Name = '_Total'" -namespace "root\CIMV2" -computername $strComputer
	Write-Host "                 Getting Win32_PerfFormattedData_PerfOS_Processor WMI data"-ForegroundColor DarkGray
	foreach($objItem in $colItems){
		$strCPUPercent = $objItem.PercentProcessorTime
		$strCPUPercent = [int]$strCPUPercent 
	}

	$colItems = get-wmiobject -class "Win32_PerfFormattedData_PerfOS_Memory" -namespace "root\CIMV2" -computername $strComputer
	Write-Host "                 Getting Win32_PerfFormattedData_PerfOS_Memory WMI data"-ForegroundColor DarkGray
	foreach($objItem in $colItems){
		$strRAMPercent = $objItem.PercentCommittedBytesInUse
		$strRAMPercent = [int]$strRAMPercent 
	}
	}
	Else
	{
	$strMFG = "Unknown"
	$strModel = "Ping Failure"
	$strMemory  = $Null
	$strProcs = $Null
	$strClockSpeed = $Null
	$strCPUPercent = $Null
	$strRAMPercent = $Null
	}
	Write-Host "         - XenApp Configuration Data..." -ForegroundColor DarkGray	

	$ZoneElection = switch ($Server.ElectionPreference) {
		"MostPreferred" {"Most Preferred"} 
        "Preferred" {"Preferred"} 
        "DefaultPreference" {"Default Preference"} 
        "NotPreferred" {"Not Preferred"} 
        default {$Server.ElectionPreference}}

	$objRSServer.AddNew()
		$objRSServer.Fields.item('AuditID').value = $AuditID
		$objRSServer.Fields.item('ServerName').value = $strComputer
		$objRSServer.Fields.item('ServerFolder').value = $Server.FolderPath
		$objRSServer.Fields.item('XenAppBuild').value = $Server.CitrixVersion
		$XenAppLoad = Get-XAServerLoad -ServerName $strComputer
			$objRSServer.Fields.item('XenAppLoad').value = $XenAppLoad.load
		$objRSServer.Fields.item('ServerIPAddress').value = $Server.IPAddresses[0] #First IP Address Only
		$LoadEvaluator = Get-XALoadEvaluator -Servername $strComputer
			$objRSServer.Fields.item('LoadEvaluator').value = $LoadEvaluator.LoadEvaluatorName
		$ServerFull = Get-XAServer -Servername $strComputer -Full
			$objRSServer.Fields.item('SessionCount').value = $ServerFull.SessionCount
		$objRSServer.Fields.item('OperatingSystem').value = $Server.OSVersion
		$objRSServer.Fields.item('ServerMake').value = $strMfg
		$objRSServer.Fields.item('ServerModel').value = $strModel
		$objRSServer.Fields.item('ServerRAM').value = $strMemory
		$objRSServer.Fields.item('ServerCPU').value = $strProcs
		$objRSServer.Fields.item('ServerCPUSpeed').value = $strClockSpeed
		$objRSServer.Fields.item('ZoneName').value = $Server.ZoneName
		$objRSServer.Fields.item('ZoneElectionPreference').value = $ZoneElection
		$ZDC = $False
			$strZone = Get-XAZone
			foreach($ZoneServer in $strZone){if ($ZoneServer.DataCollector -eq $strComputer){$ZDC = $true}}
			$objRSServer.Fields.item('IsDataCollector').value = $ZDC
		#$objRSServer.Fields.item('ServerVideoBuffer').value = objWinServer.ICAVideoBufferSize
		#$objRSServer.Fields.item('XenAppXML').value = objWinServer.XMLPortNumber
		#$objRSServer.Fields.item('ContentRedir').value = objWinServer.EnableContentRedirection
		#$objRSServer.Fields.item('XenAppVersion').value = $Server.CitrixVersion
		$objRSServer.Fields.item('XenAppEdition').value = $Server.CitrixEditionString
		$objRSServer.Fields.item('CPUPercent').value = $strCPUPercent
		$objRSServer.Fields.item('RAMPercent').value = $strRAMPercent
	$objRSServer.Update()

#ID for Child Tables	
$qryString = "Select top 1 * from tblServers where AuditID="
$qryString += $AuditID
$qryString += " ORDER BY ServerID DESC" 

#Write-Host $qryString -ForegroundColor Yellow

$objRSServer2.Open($qryString , $connection, $adOpenStatic, $adLockOptimistic)
	$objRSServer2.MoveFirst()
	$ServerID = $objRSServer2.Fields.item('ServerID').value
$objRSServer2.Close()
	
#Server Printers
Write-Host "         - Installed Printers..." -ForegroundColor DarkGray	
$objRSPrinters.Open("SELECT top 1 * FROM tblServersPrinters", $connection, $adOpenStatic, $adLockOptimistic)
	Update-XAPrinterDriver -ServerName $strComputer
	$PrintDrivers = Get-XAPrinterDriver -ServerName $strComputer
	if($PrintDrivers.count -gt 0){
	foreach($Printer in $PrintDrivers){
	$objRSPrinters.AddNew()
		$objRSPrinters.Fields.item('AuditID').value = $AuditID
		$objRSPrinters.Fields.item('ServerID').value = $ServerID
		if($XenAppVersion.startswith("6.0"))
			{$objRSPrinters.Fields.item('PrintDriver').value = $Printer
				Write-Host "                 " $Printer -ForegroundColor DarkGray}
		else
			{$objRSPrinters.Fields.item('PrintDriver').value = $Printer.DriverName
				Write-Host "                 " $Printer.DriverName -ForegroundColor DarkGray}
	$objRSPrinters.Update()	
}}
$objRSPrinters.Close()

#Server HotFixes
Write-Host "         - Installed XenApp HotFixes..." -ForegroundColor DarkGray	
$objRSHF.Open("SELECT top 1 * FROM tblServersHF", $connection, $adOpenStatic, $adLockOptimistic)
	$ServerHF = Get-XAServerHotfix -ServerName $strComputer
	if($ServerHF.count -gt 0){
	foreach($HotFix in $ServerHF){
	$objRSHF.AddNew()
		$objRSHF.Fields.item('AuditID').value = $AuditID
		$objRSHF.Fields.item('ServerID').value = $ServerID
		$objRSHF.Fields.item('HotfixName').value = $HotFix.HotfixName 
	$objRSHF.Update()	
	}}
$objRSHF.Close()

#Server Processes
Write-Host "         - Server User Processes..." -ForegroundColor DarkGray	
$objRStblProcesses.Open("SELECT top 1 * FROM tblProcesses", $connection, $adOpenStatic, $adLockOptimistic)
	$colProcesses = get-wmiobject -class "Win32_Process" -namespace "root\CIMV2" -computername $strComputer
	foreach($objProcess in $colProcesses){
		$ErrorActionPreference = "SilentlyContinue"
		[string]$ProcessUserName = $null
		$ProcessUserName = $objProcess.getowner().user
		[string]$ProcessName = $objProcess.name
		$ErrorActionPreference = "Continue"
		if ($ProcessUserName -eq "SYSTEM") {}
		elseif ($ProcessUserName -eq "LOCAL SERVICE"){}
		elseif ($ProcessUserName -eq "Ctx_StreamingSvc"){}
		elseif ($ProcessUserName -eq "") {}
		elseif ($ProcessUserName -eq "ctx_cpsvcuser") {} 
		elseif ($ProcessUserName -eq "Ctx_ConfigMgr") {}
		elseif ($ProcessUserName -eq "NETWORK SERVICE") {}
		elseif ($ProcessUserName -eq $Null) {}
		Else {
			Write-Host "                  Process: " $ProcessName " is owned by: " $ProcessUserName -ForegroundColor DarkGray
			$objRStblProcesses.AddNew()
			$objRStblProcesses.Fields.item('AuditID').value = $AuditID
			$objRStblProcesses.Fields.item('ServerID').value = $ServerID
			$objRStblProcesses.Fields.item('ServerName').value = $strComputer
			$objRStblProcesses.Fields.item('ProcessName').value = $ProcessName
			$objRStblProcesses.Fields.item('ProcessID').value = $objProcess.ProcessID
			$objRStblProcesses.Fields.item('ThreadCount').value = $objProcess.ThreadCount
			$objRStblProcesses.Fields.item('PageFileUsage').value = $objProcess.PageFileUsage
			$objRStblProcesses.Fields.item('PageFaults').value = $objProcess.PageFaults
			$objRStblProcesses.Fields.item('WorkingSetSize').value = $objProcess.WorkingSetSize
			[single]$kernelTime = $objProcess.KernelModeTime
			[single]$usertime = $objProcess.UserModeTime
			$objRStblProcesses.Fields.item('ProcessorTime').value = ($kerneltime+$usertime)/10000000
			$objRStblProcesses.Fields.item('ProcessOwner').value = $ProcessUserName
			$objRStblProcesses.Fields.item('SessionID').value = $objProcess.sessionid
			$objRStblProcesses.Fields.item('HandleCount').value = $objProcess.HandleCount
			$objRStblProcesses.Fields.item('ParentProcessID').value = $objProcess.ParentProcessID
			$objRStblProcesses.Fields.item('VirtualMemorySize').value = $objProcess.VirtualSize
			$objRStblProcesses.Update()
		}}		
$objRStblProcesses.Close()
$ErrorActionPreference = "Continue"
}
$objRSServer.Close()

Write-Host
Write-Host "Collecting Application Data..." 

$objRSApps = New-Object -com ADODB.Recordset
$objRSApps2 = New-Object -com ADODB.Recordset
$objRSAppsServers = New-Object -com ADODB.Recordset
$objRSAppsGroups = New-Object -com ADODB.Recordset
$objRSAppsUsers = New-Object -com ADODB.Recordset
$objRSAppsWorkerGroups = New-Object -com ADODB.Recordset

$objRSApps.Open("Select top 1 * from tblApps", $connection, $adOpenStatic, $adLockOptimistic)
$Applications = Get-XAApplication
foreach($app in $Applications){
	$AppName = $app.DisplayName
	$BrowserName = $app.BrowserName
	Write-Host "     " $AppName  -ForegroundColor Gray
	Write-Host "         - XenApp Application Properties..." -ForegroundColor DarkGray
	
	$ColorDepth = switch ($app.ColorDepth)     { 
        "Colors8Bit" {"8 Bit"} 
        "Colors16Bit" {"16 Bit"} 
        "Colors32Bit" {"32 Bit"} 
        default {$app.ColorDepth}    }
	
	$Audio = switch ($app.AudioType)     { 
        "None" {"None"} 
        "Basic" {"Basic"} 
        default {$app.AudioType}     }
	
	$CpuPriorityLevel = switch ($app.CpuPriorityLevel)     { 
        "BelowNormal" {"Below Normal"} 
        "Low" {"Low"} 
		"Normal" {"Normal"} 
		"AboveNormal" {"Above Normal"} 
		"High" {"High"} 
        default {$app.CpuPriorityLevel}    }
	
	$ApplicationType = switch ($app.ApplicationType)     { 
        "ServerInstalled" {"Server Installed"} 
        "ServerDesktop" {"Server Desktop"} 
		"Content" {"Content"} 
		"StreamedToServer" {"Streamed to Server"} 
		"StreamedToClient" {"Streamed to Client"} 
		"StreamedToClientOrInstalled" {"Streamed to Client or Installed"} 
		"StreamedToClientOrStreamedToServer" {"Streamed to Client or Streamed to Server"} 
        default {$app.ApplicationType}    }
	
	$EncryptionLevel = switch ($app.EncryptionLevel) { 
        "Basic" {"Basic"} 
        "LogOn" {"Logon Only"} 
		"Bits40" {"40-bit"} 
		"Bits56" {"56-bit"} 
		"Bits128" {"128-bit"} 
        default {$app.EncryptionLevel}   }	
	
	$objRSApps.AddNew()
		$objRSApps.Fields.item('AuditID').value = $AuditID
		$objRSApps.Fields.item('AppName').value = $AppName
		$objRSApps.Fields.item('BrowserName').value = $BrowserName 
		$objRSApps.Fields.item('AppCommandLine').value = $app.CommandLineExecutable
		#ContentAddress Logic for Command Line		
		$objRSApps.Fields.item('AppWorkingDir').value = $app.WorkingDirectory
		$AppDN = $app.FolderPath+"/"+$app.AppName
			$objRSApps.Fields.item('AppDN').value = $AppDN
		$objRSApps.Fields.item('WindowType').value = $app.WindowType 
		$objRSApps.Fields.item('ColorDepth').value = $ColorDepth
		$objRSApps.Fields.item('AudioQuality').value = $Audio
		$objRSApps.Fields.item('Encryption').value = $EncryptionLevel
		$objRSApps.Fields.item('Folder').value = $app.FolderPath
		$Disabled = if($app.Enabled){$False} else {$True}
			$objRSApps.Fields.item('Disabled').value = $Disabled
		$objRSApps.Fields.item('PNAgentFolder').value = $app.ClientFolder
		$objRSApps.Fields.item('AppType').value = $ApplicationType
		$objRSApps.Fields.item('Priority').value = $CpuPriorityLevel
	$objRSApps.Update()

#ID for Child Tables	
$qryString = "Select top 1 * from tblApps where AuditID="
$qryString += $AuditID
$qryString += " ORDER BY AppID DESC" 

$objRSApps2.Open($qryString , $connection, $adOpenStatic, $adLockOptimistic)
	$objRSApps2.MoveFirst()
	$AppID = $objRSApps2.Fields.item('AppID').value
$objRSApps2.Close()

$AppReport = get-XAApplicationReport $BrowserName
		
#App Servers
Write-Host "         - Application Server Assignments..." -ForegroundColor DarkGray	
$objRSAppsServers.Open("SELECT top 1 * FROM tblAppsServers", $connection, $adOpenStatic, $adLockOptimistic)
	$ascount = $AppReport.ServerNames
	if($ascount.count -gt 0){
	foreach($AppServer in $AppReport.ServerNames){
	$objRSAppsServers.AddNew()
		$objRSAppsServers.Fields.item('AuditID').value = $AuditID
		$objRSAppsServers.Fields.item('AppID').value = $AppID
		$objRSAppsServers.Fields.item('AppServerName').value = $AppServer
	$objRSAppsServers.Update()	
	}}
$objRSAppsServers.Close()

#App WorkerGroups
Write-Host "         - Application Worker Group Assignments..." -ForegroundColor DarkGray	
$objRSAppsWorkerGroups.Open("SELECT top 1 * FROM tblAppsWorkerGroups", $connection, $adOpenStatic, $adLockOptimistic)
	$awgcount = $AppReport.WorkerGroupNames
	if($awgcount.count -gt 0){
	foreach($AppWG in $AppReport.WorkerGroupNames){
	$objRSAppsWorkerGroups.AddNew()
		$objRSAppsWorkerGroups.Fields.item('AuditID').value = $AuditID
		$objRSAppsWorkerGroups.Fields.item('AppID').value = $AppID
		$objRSAppsWorkerGroups.Fields.item('WorkerGroup').value = $AppWG
	$objRSAppsWorkerGroups.Update()	
	}}
$objRSAppsWorkerGroups.Close()

#App Users & Groups
Write-Host "         - Application Account Assignments..." -ForegroundColor DarkGray	
$objRSAppsGroups.Open("SELECT top 1 * FROM tblAppsGroups", $connection, $adOpenStatic, $adLockOptimistic)
$objRSAppsUsers.Open("SELECT top 1 * FROM tblAppsUsers", $connection, $adOpenStatic, $adLockOptimistic)
$appassignedusers = get-xaaccount -browsername $BrowserName
	foreach($AccessAccount in $appassignedusers){
		if ($AccessAccount.AccountType -eq 4)   #Group
		{$objRSAppsGroups.AddNew()
			$objRSAppsGroups.Fields.item('AuditID').value = $AuditID
			$objRSAppsGroups.Fields.item('AppID').value = $AppID
			$objRSAppsGroups.Fields.item('GroupName').value = $AccessAccount.AccountDisplayName 
		$objRSAppsGroups.Update()}
		if ($AccessAccount.AccountType -eq 1)   #User
		{$objRSAppsUsers.AddNew()
			$objRSAppsUsers.Fields.item('AuditID').value = $AuditID
			$objRSAppsUsers.Fields.item('AppID').value = $AppID
			$objRSAppsUsers.Fields.item('UserName').value = $AccessAccount.AccountDisplayName 
		$objRSAppsUsers.Update()}
	}
	$objRSAppsGroups.close()
	$objRSAppsUsers.close()
}
$objRSApps.Close() 

#WorkerGroups
Write-Host
Write-Host "Collecting Worker Group Data..." 
$objRSWorkerGroups = New-Object -com ADODB.Recordset
$objRSWorkerGroup2 = New-Object -com ADODB.Recordset
$objRSWorkerGroupsServers = New-Object -com ADODB.Recordset
$objRSWorkerGroupsSGs = New-Object -com ADODB.Recordset
$objRSWorkerGroupsOU = New-Object -com ADODB.Recordset
$objRSWorkerGroups.Open("Select top 1 * from tblWorkerGroups", $connection, $adOpenStatic, $adLockOptimistic)
$objRSWorkerGroupsServers.Open("Select top 1 * from tblWorkerGroupsServers", $connection, $adOpenStatic, $adLockOptimistic)
$objRSWorkerGroupsSGs.Open("Select top 1 * from tblWorkerGroupsSG", $connection, $adOpenStatic, $adLockOptimistic)
$objRSWorkerGroupsOU.Open("Select top 1 * from tblWorkerGroupsOU", $connection, $adOpenStatic, $adLockOptimistic)
$WGs = Get-XAWorkerGroup
foreach($WG in $WGs){
	$objRSWorkerGroups.AddNew()
		$objRSWorkerGroups.Fields.item('AuditID').value = $AuditID
		$objRSWorkerGroups.Fields.item('WorkerGroup').value = $WG.WorkerGroupName
		$objRSWorkerGroups.Fields.item('Description').value = $WG.Description
		$objRSWorkerGroups.Fields.item('FolderPath').value = $WG.FolderPath
	$objRSWorkerGroups.Update()
	
	#ID for Child Tables	
	$qryString = "Select top 1 * from tblWorkerGroups where AuditID="
	$qryString += $AuditID
	$qryString += " ORDER BY WorkerGroupID DESC" 
	
	$objRSWorkerGroup2.Open($qryString , $connection, $adOpenStatic, $adLockOptimistic)
		$objRSWorkerGroup2.MoveFirst()
	$WorkerGroupID = $objRSWorkerGroup2.Fields.item('WorkerGroupID').value
	$objRSWorkerGroup2.Close()
	
	foreach($wgserver in $wg.servernames){
	$objRSWorkerGroupsServers.AddNew()
		$objRSWorkerGroupsServers.Fields.item('AuditID').value = $AuditID
		$objRSWorkerGroupsServers.Fields.item('WorkerGroupID').value = $WorkerGroupID
		$objRSWorkerGroupsServers.Fields.item('ServerName').value = $wgserver
	$objRSWorkerGroupsServers.Update()}
	
	foreach($wgsvrgrp in $wg.servergroups){
	$objRSWorkerGroupsSGs.AddNew()
		$objRSWorkerGroupsSGs.Fields.item('AuditID').value = $AuditID
		$objRSWorkerGroupsSGs.Fields.item('WorkerGroupID').value = $WorkerGroupID
		$objRSWorkerGroupsSGs.Fields.item('ServerGroup').value = $wgsrvgrp
	$objRSWorkerGroupsSGs.Update()}

	foreach($wgou in $wg.OUs){
	$objRSWorkerGroupsOU.AddNew()
		$objRSWorkerGroupsOU.Fields.item('AuditID').value = $AuditID
		$objRSWorkerGroupsOU.Fields.item('WorkerGroupID').value = $WorkerGroupID
		$objRSWorkerGroupsOU.Fields.item('OUName').value = $wgou
	$objRSWorkerGroupsOU.Update()}	
}
$objRSWorkerGroupsOU.Close()
$objRSWorkerGroupsSGs.Close()
$objRSWorkerGroupsServers.Close()
$objRSWorkerGroups.Close()


#Admins
Write-Host
Write-Host "Collecting Farm Administrators Data..." 
$objRSAdmins = New-Object -com ADODB.Recordset
$objRSAdmins.Open("Select top 1 * from tblMFAdmins", $connection, $adOpenStatic, $adLockOptimistic)
$mfadmins = Get-XAAdministrator
foreach($mfadmin in $mfadmins){
	$MFAdminType = switch ($mfadmin.AdministratorType){ 
        "Full" {"Full"} 
        "ViewOnly" {"View Only"} 
        "Custom" {"Custom"} 
        default {$mfadmin.AdministratorType}}
		
	$objRSAdmins.AddNew()
		$objRSAdmins.Fields.item('AuditID').value = $AuditID
		$objRSAdmins.Fields.item('MFAdminName').value = $mfadmin.AdministratorName
		$objRSAdmins.Fields.item('MFAdminRole').value = $MFAdminType
	$objRSAdmins.Update()
}
$objRSAdmins.close()

#Load Evaluators
Write-Host
Write-Host "Collecting Load Evaluator Data..." 
$objRSLoadEval = New-Object -com ADODB.Recordset
$objRSLoadEval2 = New-Object -com ADODB.Recordset
$objRSLoadEvalRules = New-Object -com ADODB.Recordset
$objRSLoadEval.Open("Select top 1 * from tblLoadEvals", $connection, $adOpenStatic, $adLockOptimistic)
$XALE = Get-XALoadEvaluator
foreach($LE in $XALE){
	$objRSLoadEval.AddNew()
		$objRSLoadEval.Fields.item('AuditID').value = $AuditID
		$objRSLoadEval.Fields.item('LENAme').value = $LE.LoadEvaluatorName
		$objRSLoadEval.Fields.item('LEDesc').value = $LE.Description
	$objRSLoadEval.Update()
		
#ID for Child Tables	
$qryString = "Select top 1 * from tblLoadEvals where AuditID="
$qryString += $AuditID
$qryString += " ORDER BY LoadEvalID DESC" 

$objRSLoadEval2.Open($qryString , $connection, $adOpenStatic, $adLockOptimistic)
	$objRSLoadEval2.MoveFirst()
	$LoadEvalID = $objRSLoadEval2.Fields.item('LoadEvalID').value
$objRSLoadEval2.Close()

$objRSLoadEvalRules.Open("Select top 1 * from tblLoadEvalsRules", $connection, $adOpenStatic, $adLockOptimistic)		
	if($LE.ApplicationUserLoadEnabled){
		$objRSLoadEvalRules.AddNew()
			$objRSLoadEvalRules.Fields.item('AuditID').value = $AuditID
			$objRSLoadEvalRules.Fields.item('LoadEvalID').value = $LoadEvalID
			$objRSLoadEvalRules.Fields.item('LERuleType').value = "Application User Load"
			$objRSLoadEvalRules.Fields.item('LERuleHWM').value = $LE.ApplicationUserLoad
			$objRSLoadEvalRules.Fields.item('LERuleLWM').value = $null
		$objRSLoadEvalRules.Update()}
	if($LE.ContextSwitchesEnabled){
		$objRSLoadEvalRules.AddNew()
			$objRSLoadEvalRules.Fields.item('AuditID').value = $AuditID
			$objRSLoadEvalRules.Fields.item('LoadEvalID').value = $LoadEvalID
			$objRSLoadEvalRules.Fields.item('LERuleType').value = "Context Switches"
			$objRSLoadEvalRules.Fields.item('LERuleHWM').value = $LE.ContextSwitches[1]
			$objRSLoadEvalRules.Fields.item('LERuleLWM').value = $LE.ContextSwitches[0]
		$objRSLoadEvalRules.Update()}
	if($LE.CpuUtilizationEnabled){
		$objRSLoadEvalRules.AddNew()
			$objRSLoadEvalRules.Fields.item('AuditID').value = $AuditID
			$objRSLoadEvalRules.Fields.item('LoadEvalID').value = $LoadEvalID
			$objRSLoadEvalRules.Fields.item('LERuleType').value = "CPU Utilization"
			$objRSLoadEvalRules.Fields.item('LERuleHWM').value = $LE.CpuUtilization[1]
			$objRSLoadEvalRules.Fields.item('LERuleLWM').value = $LE.CpuUtilization[0]
		$objRSLoadEvalRules.Update()}
	if($LE.DiskDataIOEnabled){
		$objRSLoadEvalRules.AddNew()
			$objRSLoadEvalRules.Fields.item('AuditID').value = $AuditID
			$objRSLoadEvalRules.Fields.item('LoadEvalID').value = $LoadEvalID
			$objRSLoadEvalRules.Fields.item('LERuleType').value = "Disk Data IO"
			$objRSLoadEvalRules.Fields.item('LERuleHWM').value = $LE.DiskDataIO[1]
			$objRSLoadEvalRules.Fields.item('LERuleLWM').value = $LE.DiskDataIO[0]
		$objRSLoadEvalRules.Update()}
	if($LE.DiskOperationsEnabled){
		$objRSLoadEvalRules.AddNew()
			$objRSLoadEvalRules.Fields.item('AuditID').value = $AuditID
			$objRSLoadEvalRules.Fields.item('LoadEvalID').value = $LoadEvalID
			$objRSLoadEvalRules.Fields.item('LERuleType').value = "Disk Operations"
			$objRSLoadEvalRules.Fields.item('LERuleHWM').value = $LE.DiskOperations[1]
			$objRSLoadEvalRules.Fields.item('LERuleLWM').value = $LE.DiskOperations[0]
		$objRSLoadEvalRules.Update()}
	if($LE.IPRangesEnabled){
		$objRSLoadEvalRules.AddNew()
			$objRSLoadEvalRules.Fields.item('AuditID').value = $AuditID
			$objRSLoadEvalRules.Fields.item('LoadEvalID').value = $LoadEvalID
			$objRSLoadEvalRules.Fields.item('LERuleType').value = "IP Ranges"
			$objRSLoadEvalRules.Fields.item('LERuleHWM').value = $LE.IPRanges[-1]
			$objRSLoadEvalRules.Fields.item('LERuleLWM').value = $LE.IPRanges[0]
		$objRSLoadEvalRules.Update()}
	if($LE.LoadThrottlingEnabled){
		$objRSLoadEvalRules.AddNew()
			$objRSLoadEvalRules.Fields.item('AuditID').value = $AuditID
			$objRSLoadEvalRules.Fields.item('LoadEvalID').value = $LoadEvalID
			$objRSLoadEvalRules.Fields.item('LERuleType').value = "Load Throttling"
			$objRSLoadEvalRules.Fields.item('LERuleHWM').value = $LE.LoadThrottling
			$objRSLoadEvalRules.Fields.item('LERuleLWM').value = $NULL
		$objRSLoadEvalRules.Update()}
	if($LE.MemoryUsageEnabled){
		$objRSLoadEvalRules.AddNew()
			$objRSLoadEvalRules.Fields.item('AuditID').value = $AuditID
			$objRSLoadEvalRules.Fields.item('LoadEvalID').value = $LoadEvalID
			$objRSLoadEvalRules.Fields.item('LERuleType').value = "Memory Usage"
			$objRSLoadEvalRules.Fields.item('LERuleHWM').value = $LE.MemoryUsage[1]
			$objRSLoadEvalRules.Fields.item('LERuleLWM').value = $LE.MemoryUsage[0]
		$objRSLoadEvalRules.Update()}
	if($LE.PageFaultsEnabled){
		$objRSLoadEvalRules.AddNew()
			$objRSLoadEvalRules.Fields.item('AuditID').value = $AuditID
			$objRSLoadEvalRules.Fields.item('LoadEvalID').value = $LoadEvalID
			$objRSLoadEvalRules.Fields.item('LERuleType').value = "Page Faults"
			$objRSLoadEvalRules.Fields.item('LERuleHWM').value = $LE.PageFaults[1]
			$objRSLoadEvalRules.Fields.item('LERuleLWM').value = $LE.PageFaults[0]
		$objRSLoadEvalRules.Update()}
	if($LE.PageSwapsEnabled){
		$objRSLoadEvalRules.AddNew()
			$objRSLoadEvalRules.Fields.item('AuditID').value = $AuditID
			$objRSLoadEvalRules.Fields.item('LoadEvalID').value = $LoadEvalID
			$objRSLoadEvalRules.Fields.item('LERuleType').value = "Page Swaps"
			$objRSLoadEvalRules.Fields.item('LERuleHWM').value = $LE.PageSwaps[1]
			$objRSLoadEvalRules.Fields.item('LERuleLWM').value = $LE.PageSwaps[0]
		$objRSLoadEvalRules.Update()}
	if($LE.ScheduleEnabled){
		$objRSLoadEvalRules.AddNew()
			$objRSLoadEvalRules.Fields.item('AuditID').value = $AuditID
			$objRSLoadEvalRules.Fields.item('LoadEvalID').value = $LoadEvalID
			$objRSLoadEvalRules.Fields.item('LERuleType').value = "Schedule Enabled"
			$objRSLoadEvalRules.Fields.item('LERuleHWM').value = $NULL
			$objRSLoadEvalRules.Fields.item('LERuleLWM').value = $NULL
		$objRSLoadEvalRules.Update()}		
	if($LE.ServerUserLoadEnabled){
		$objRSLoadEvalRules.AddNew()
			$objRSLoadEvalRules.Fields.item('AuditID').value = $AuditID
			$objRSLoadEvalRules.Fields.item('LoadEvalID').value = $LoadEvalID
			$objRSLoadEvalRules.Fields.item('LERuleType').value = "Server User Load"
			$objRSLoadEvalRules.Fields.item('LERuleHWM').value = $LE.ServerUserLoad
			$objRSLoadEvalRules.Fields.item('LERuleLWM').value = $NULL
		$objRSLoadEvalRules.Update()}				
$objRSLoadEvalRules.Close()		
}
$objRSLoadEval.Close()


#Sessions
Write-Host
Write-Host "Collecting Session Data..." 
$objRSSessions = New-Object -com ADODB.Recordset
$objRSSessions.Open("Select top 1 * from tblSessions", $connection, $adOpenStatic, $adLockOptimistic)
$Sessions = Get-XASession -Farm -Full
foreach($Session in $Sessions){
	$SessionState = switch ($Session.State) 
    { 
        "Active" {"Active"} 
        "Connected" {"Connected"} 
		"Connecting" {"Connecting"} 
		"Shadowing" {"Shadowing"} 
		"Disconnected" {"Disconnected"}
        "Idle" {"Idle"} 
        "Listening" {"Listening"} 
		"Resetting" {"Resetting"} 
		"Down" {"Down"} 
		"Initializing" {"Initializing"}		
        "Stale" {"Stale"} 
		"Licensed" {"Licensed"} 
		"Unlicensed" {"Unlicensed"} 
		"Reconnected" {"Reconnected"}
        default {$Session.State}
    }	
	$objRSSessions.AddNew()
		$objRSSessions.Fields.item('AuditID').value = $AuditID
		$objRSSessions.Fields.item('UserName').value = $Session.AccountName 
		$objRSSessions.Fields.item('ClientBuild').value = $Session.ClientBuildNumber
		$objRSSessions.Fields.item('ClientName').value = $Session.ClientName
		$objRSSessions.Fields.item('ServerName').value = $Session.ServerName
		$objRSSessions.Fields.item('SessionID').value = $Session.SessionID
		$objRSSessions.Fields.item('SessionName').value = $Session.SessionName
		$objRSSessions.Fields.item('SessionState').value = $SessionState
		$objRSSessions.Fields.item('LogonTime').value = $Session.ConnectTime
	$objRSSessions.Update()
}
$objRSSessions.close()


$connection.Close() 
$EndTime = Get-Date -Format G
Write-Host
Write-Host "-----------------------------------------------------------" -ForegroundColor Red
Write-Host "Audit #" $AuditID "Completed at" $EndTime -ForegroundColor Red
Write-Host "-----------------------------------------------------------" -ForegroundColor Red
Write-Host
stop-transcript 
#Write-Host "Press any key to continue ..." -ForegroundColor Blue
#Write-Host
#$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")